using System;
using System.IO;

using pdftron;
using pdftron.PDF;
using pdftron.PDF.Annots;
using pdftron.SDF;
using pdftron.Crypto;

namespace DigitalSignaturesTestCS
{

	class Class1
	{
		static string input_path = "../../../../TestFiles/";
		static string output_path = "../../../../TestFiles/Output/";

		static void SignAndTSAPDF(
			string in_approval_field_name,
			string in_private_key_file_path,
			string in_keyfile_password,
			string in_appearance_img_path,
			string in_timestamp_authority_root_certificate_path,
			string in_outpath)
		{
			Console.Out.WriteLine("================================================================================");
			Console.Out.WriteLine("Signing PDF document");

			// Open an existing PDF
			using (PDFDoc doc = new PDFDoc())
			{
				var page = doc.PageCreate();
				doc.PagePushBack(page);

				// Create a digital signature field and associated widget.
				DigitalSignatureField digsig_field = doc.CreateDigitalSignatureField(in_approval_field_name);
				SignatureWidget widgetAnnot = SignatureWidget.Create(doc, new Rect(143, 287, 219, 306), digsig_field);
				Image img = Image.Create(doc, in_appearance_img_path);
				widgetAnnot.CreateSignatureAppearance(img);
				page.AnnotPushBack(widgetAnnot);

				// Create a digital signature dictionary inside the digital signature field, in preparation for signing.
				digsig_field.CreateSigDictForCustomSigning("Adobe.PPKLite", DigitalSignatureField.SubFilterType.e_adbe_pkcs7_detached,
					8200); // For security reasons, set the contents size to a value greater than but as close as possible to the size you expect your final signature to be, in bytes.

				// Save the document incrementally to avoid invalidating any previous signatures.
				doc.Save(in_outpath, SDFDoc.SaveOptions.e_incremental);

				// Digest the relevant bytes of the document in accordance with ByteRanges surrounding the signature.
				byte[] pdf_digest = digsig_field.CalculateDigest(DigestAlgorithm.Type.e_sha256);

				byte[] in_pkcs12_buffer = File.ReadAllBytes(in_private_key_file_path);
				byte[] signature_value = DigitalSignatureField.SignDigest(pdf_digest, in_pkcs12_buffer, in_keyfile_password, false, DigestAlgorithm.Type.e_sha256);

				// Write the signature to the document.
				doc.SaveCustomSignature(signature_value, digsig_field, in_outpath);

                // Add embedded timestamp to signature.
                TimestampingConfiguration tst_config = new TimestampingConfiguration("http://rfc3161timestamp.globalsign.com/advanced");
                VerificationOptions opts = new VerificationOptions(VerificationOptions.SignatureVerificationSecurityLevel.e_compatibility_and_archiving);
                opts.AddTrustedCertificate(in_timestamp_authority_root_certificate_path);
                opts.EnableOnlineCRLRevocationChecking(true);
                TimestampingResult result = digsig_field.GenerateContentsWithEmbeddedTimestamp(tst_config, opts);

                if (!result.GetStatus())
                {
                    Console.WriteLine(result.GetString());
                    throw new Exception();
                }
                doc.SaveCustomSignature(result.GetData(), digsig_field, in_outpath);

            }
		}

		private static pdftron.PDFNetLoader pdfNetLoader = pdftron.PDFNetLoader.Instance();
		static Class1() { }

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			// Initialize PDFNetC
			PDFNet.Initialize(PDFTronLicense.Key);

			bool result = true;

			// Add approval signature and time stamp
			try
			{
				SignAndTSAPDF(
					"PDFTronCertificationSig",
					input_path + "pdftron.pfx",
					"password",
					input_path + "pdftron.bmp",
					input_path + "GlobalSignRootForTST.cer",
					output_path + "waiver_withApprovalField_certified_output_TSA_01.pdf");
			}
			catch (Exception e)
			{
				Console.Error.WriteLine(e);
				result = false;
			}

			//////////////////// End of tests. ////////////////////
			PDFNet.Terminate();
			if (result)
			{
				Console.Out.WriteLine("Tests successful.\n==========");
			}
			else
			{
				Console.Out.WriteLine("Tests FAILED!!!\n==========");
			}
		}
	}
}
